#include <assert.h>
#include "Observable.h"


/*
 * This object implements the Observable interface as part of the 
 * Observer/Observable design pattern.
 * All observers are added as pointers to objects which implemented the
 * Observer interface.
 * Note: This implementation uses STL (Standard Template Library).
 *
 * Author: Ziv Yaniv (zivy@cs.huji.ac.il)
 */


using namespace std;

void Observable::addObserver(Observer *o) 
{
  assert(o != NULL); //this should never happen

             //multiple insertions of the same observer are not allowed
	std::list<Observer *>::iterator it = _observers.begin();
	std::list<Observer *>::iterator end = _observers.end();
  short found = 0;

  while((it != end) && !found) {
    if((*it) == o)
      found = 1;
    else it++;
  }

  if(it == end)
    _observers.push_back(o);
}
/**********************************************************************/
void Observable::deleteObserver(Observer *o)
{
	std::list<Observer *>::iterator it = _observers.begin();
	std::list<Observer *>::iterator end = _observers.end();
  short found = 0;

  while((it != end) && !found) {
    if((*it) == o)
      found = 1;
    else it++;
  }

  if(it != end)
    _observers.erase(it);
}
/**********************************************************************/
void Observable::deleteObservers()
{
  _observers.erase(_observers.begin(),_observers.end());
}
/**********************************************************************/
void Observable::notifyObservers() 
{
	std::list<Observer *>::iterator end = _observers.end();
  
  for(list<Observer *>::iterator it=_observers.begin() ; it != end ; it++) 
    (*it)->update(this,NULL);
  clearChanged();
}
/**********************************************************************/
void Observable::notifyObservers(void *userData) 
{
	std::list<Observer *>::iterator end = _observers.end();
  
  for(list<Observer *>::iterator it=_observers.begin() ; it != end ; it++) 
    (*it)->update(this,userData);
  clearChanged();
}


