#ifndef _OBSERVABLE_H_
#define _OBSERVABLE_H_

#include <list>
#include "Observer.h"


/*
 * This object implements the Observable interface as part of the 
 * Observer/Observable design pattern.
 * All observers are added as pointers to objects which implemented the
 * Observer interface.
 * Note: This implementation uses STL (Standard Template Library).
 *
 * Author: Ziv Yaniv (zivy@cs.huji.ac.il)
 */

class Observable {
public:
  Observable() : _hasChanged(0) {}

              //adds an observer to the set of observers for this object. 
              //multiple insertions of the same observer will be ignored.
  void addObserver(Observer *o);

            //indicates that this object has no longer changed, 
           //or that it has already notified all of its observers of 
          //its most recent change. 
  void clearChanged() { _hasChanged=0; } 

           //returns the number of observers this object has. 
  unsigned countObservers() { return(_observers.size()); }

        //deletes an observer from the set of observers this object has. 
  void deleteObserver(Observer *o); 

        //clears the observer list so that this object no longer has any 
       //observers. 
  void deleteObservers();

         //tests if this object has changed. 
  short hasChanged() { return(_hasChanged == 1); }
 
             //if this object has changed, as indicated by the hasChanged 
            //method, then notify all of its observers and then call the 
           //clearChanged method to indicate that this object has no longer
          //changed. 
  void notifyObservers();

          //notify the observers and pass them an opaque data structure
         //according to what they expect
  void notifyObservers(void *userData); 

      //indicate that this object has changed. 
  void setChanged() { _hasChanged = 1; } 
 
private:
	std::list<Observer *> _observers; //STL container
  short _hasChanged;
};

#endif //_OBSERVABLE_H_

